//
//  EventStore.swift
//  UkiCalendar
//
//  Created by ukiLab on 2024/01/22.
//

import Foundation
import SQLite
import SwiftUI

class EventStore {
  static let DIR_SC_DB = "ukicalendar"
  static let STORE_NAME = "ukicalendar.db"

  private let events = Table("events")

  private let id = Expression<Int64>("id")
  private let title = Expression<String>("title")
  private let startDate = Expression<Date>("start_date")
  private let endDate = Expression<Date>("end_date")
  private let isAllDay = Expression<Bool>("is_all_day")
  private let note = Expression<String>("note")
  private let colorComponents = Expression<String>("color_components")


  static let shared = EventStore()

  private var db: Connection? = nil

  private init() {
    if let docDir = FileManager.default.urls(for: .documentDirectory, in: .userDomainMask).first {
      let dirPath = docDir.appendingPathComponent(Self.DIR_SC_DB)
      debugLog("dicDir: \(docDir)")

      do {
        try FileManager.default.createDirectory(atPath: dirPath.path, withIntermediateDirectories: true, attributes: nil)
        let dbPath = dirPath.appendingPathComponent(Self.STORE_NAME).path
        db = try Connection(dbPath)
        createTable()
        debugLog("SQLiteDataStore init successfully at: \(dbPath) ")
      } catch {
        db = nil
        debugLog("SQLiteDataStore init error: \(error)")
      }
    } else {
      db = nil
    }
  }

  private func createTable() {
    guard let database = db else {
      return
    }
    do {
      try database.run(events.create(ifNotExists: true) { table in
        table.column(id, primaryKey: .autoincrement)
        table.column(title)
        table.column(startDate)
        table.column(endDate)
        table.column(isAllDay)
        table.column(note)
        table.column(colorComponents)
      })
      debugLog("Table Created.")
    } catch {
      debugLog(error.localizedDescription)
    }
  }

  func getAllEvents() async -> [EventData] {
    var events: [EventData] = []
    guard let database = db else { return [] }

    do {
      for event in try database.prepare(self.events) {
        events.append(
          EventData(id: event[id]
                    , title: event[title]
                    , startDate: event[startDate]
                    , endDate: event[endDate]
                    , isAllDay: event[isAllDay]
                    , note: event[note]
                    , color: decodeColor(event[colorComponents])
                   )
        )
      }
    } catch {
      debugLog(error.localizedDescription)
    }
    return events
  }

  private func decodeColor(_ colorStr: String) -> Color {
    if !colorStr.isEmpty {
      let colorComponents = colorStr.split(separator: ",")
      if (colorComponents.count == 4) {
        return Color(UIColor(red: Double(colorComponents[0])!, green: Double(colorComponents[1])!, blue: Double(colorComponents[2])!, alpha: Double(colorComponents[3])!))
      }
    }
    return Constants.eventColorDefault
  }

  private func encodeColor(_ color: Color?) -> String {
    if let cc = color {
      let uiColor = UIColor(cc).cgColor
      if let components = uiColor.components {
        return components.map { "\(String(format: "%.6f", $0))" }.joined(separator: ",")
      }
    }
    return Constants.eventColorStringDefault
  }

  func createEvent(event: EventData) -> Int64? {
    guard let database = db else { return nil }

    let insert = events.insert(title <- event.title
                               , startDate <- event.startDate
                               , endDate <- event.endDate ?? event.startDate
                               , isAllDay <- event.isAllDay
                               , note <- event.note ?? ""
                               , colorComponents <- encodeColor(event.color)
    )
    do {
      let rowID = try database.run(insert)
      return rowID
    } catch {
      debugLog(error.localizedDescription)
      return nil
    }
  }

  func findEvent(eventId: Int64) -> EventData? {
    var event: EventData = EventData(id: eventId, startDate: Date())
    guard let database = db else { return nil }

    let filter = events.filter(id == eventId)
    do {
      for t in try database.prepare(filter) {
        event.title = t[title]
        event.startDate = t[startDate]
        event.endDate = t[endDate]
        event.isAllDay = t[isAllDay]
        event.note = t[note]
        event.color = decodeColor(t[colorComponents])
      }
    } catch {
      debugLog(error.localizedDescription)
    }
    return event
  }

  func updateEvent(event: EventData) -> Bool {
    guard let database = db else { return false }
    guard let eventId = event.id else { return false }

    let targetEvent = events.filter(id == eventId)
    let update = targetEvent.update(title <- event.title
                                    , startDate <- event.startDate
                                    , endDate <- event.endDate ?? event.startDate
                                    , isAllDay <- event.isAllDay
                                    , note <- event.note ?? ""
                                    , colorComponents <- encodeColor(event.color)
    )
    do {
      let count = try database.run(update)
      return count > 0
    } catch {
      debugLog(error.localizedDescription)
      return false
    }
  }

  func deleteEvent(event: EventData) -> Bool {
    guard let database = db else { return false }
    guard let eventId = event.id else { return false }

    let targetEvent = events.filter(id == eventId)
    do {
      let count = try database.run(targetEvent.delete())
      return count > 0
    } catch {
      debugLog(error.localizedDescription)
      return false
    }
  }

}
