//
//  ContentView.swift
//  UkiCalendar
//
//  Created by ukiLab on 2023/10/23.
//

import SwiftUI

struct ContentView: View {
  @State var currentDate: Date = Date()
  @EnvironmentObject var calendarDate: CalendarDate
  @EnvironmentObject var eventList: EventList
  @State private var pageIndex = 0
  @State private var isNewEventFormPresented: Bool = false
  @State private var isEventListViewPresented: Bool = false
  @State private var currentEvent: EventData?
  
  var body: some View {
    ZStack {
      VStack {
        NavigationView {
          VStack {
            InfinitePageView(
              selection: $pageIndex,
              before: { $0 - 1 },
              after: { $0 + 1 },
              view: { index in
                let targetDate = Calendar.current.date(byAdding: .month, value: index, to: currentDate) ?? Date()
                MonthView(currentDate: targetDate)
                  .id(index)
                // セルのタップイベント
                // 同じセルタップしても動作するようにUUIDを利用
                  .onChange(of: calendarDate.uuid) {
                    if (index != pageIndex) {
                      return
                    }
                    debugLog("page index: \(pageIndex)")
                    debugLog("selectedDate: \(calendarDate.selectedDate.dateTimeString())")
                    debugLog("lastSelectedDate: \(calendarDate.lastSelectedDate.dateTimeString())")
                    if (calendarDate.selectedDate.yearMonth() != calendarDate.showingDate.yearMonth()) {
                      updateMonth()
                    } else if (calendarDate.selectedDate.dateString() == calendarDate.lastSelectedDate.dateString()) {
                      isEventListViewPresented.toggle()
                    }
                  }
                // スワイプしたらヘッダの表示が当月に変わるよう
                  .onChange(of: pageIndex, { oldValue, newValue in
                    if (index != pageIndex) {
                      return
                    }
                    calendarDate.lastSelectedDate = targetDate
                    calendarDate.showingDate = targetDate
                  })
              }
            )
          }
          .navigationBarTitle(calendarDate.showingDate.yearMonthString(), displayMode: .inline)
          .navigationBarItems(leading: Button(action: {
            debugLog("menu")
          }) {
            Text("Menu")
          }, trailing:  HStack {
            Button(action: {
              debugLog("today")
              pageIndex = 0
              calendarDate.selectedDate = Date()
            }) {
              Text("Today")
            }
          })
        }
        .task {
          do {
            try await eventList.loadEvents()
          } catch {
            debugLog(error.localizedDescription)
          }
        }
        // イベント登録フォーム
        .sheet(isPresented: $isNewEventFormPresented, onDismiss: createEventIfExist) {
          // イベント登録フォーム生成
          EventFormView(event: $currentEvent)
        }
        // イベントリスト
        .sheet(isPresented: $isEventListViewPresented, onDismiss: createEventIfExist) {
          EventListView(isNewEventPressed: $isNewEventFormPresented)
        }
      }
      FloatingButton(trigger: $isNewEventFormPresented)
    }
  }
  private func updateMonth() {
    debugLog("year month: \(calendarDate.selectedDate.yearMonth()), last: \(calendarDate.lastSelectedDate.yearMonth())")
    if (calendarDate.selectedDate.yearMonth() < calendarDate.lastSelectedDate.yearMonth()) {
      debugLog("last month")
      pageIndex -= 1
    } else if (calendarDate.selectedDate.yearMonth() > calendarDate.lastSelectedDate.yearMonth()) {
      debugLog("next month")
      pageIndex += 1
    }
    calendarDate.lastSelectedDate = calendarDate.selectedDate
  }
  
  private func createEventIfExist() {
    if let event = currentEvent {
      debugLog(event.title)
      if let id = EventStore.shared.createEvent(event: event) {
        if let newEvent = EventStore.shared.findEvent(eventId: id) {
          eventList.addEvent(newEvent)
        }
      }
    }
    currentEvent = nil
  }
}

#Preview {
  ContentView().environmentObject(CalendarDate())
    .environmentObject(EventList())
  
}
